/*
 * ﻿Copyright (C) 2011-2012 NewMain Softech
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package com.newmainsoftech.aspectjutil.eventmanager.label;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import com.newmainsoftech.aspectjutil.eventmanager.EventHandler;
import com.newmainsoftech.aspectjutil.eventmanager.SubscriberMethodUnit;

/**
 * Method annotated with this will be invoked before any event in interest happens. <br />
 * Class (not interface) owning such method must be annotated with {@link EventListener} as well. <br />
 * CUATION: this will be be treated as being INHERITED.
 * <br />
 * For usage example, see {@link EventHandler} JavaDoc.
 * 
 * @author <a href="mailto:artymt@gmail.com">Arata Y.</a>
 */
@Retention( RetentionPolicy.RUNTIME)
@Target( ElementType.METHOD)
public @interface PreEvent {
	/**
	 * Array of events in interest
	 * @return events in interest
	 */
	public Class<?>[] value();
	
	/**
	 * Specify whether interesting in event triggered only within same thread what 
	 * event listener object has been constructed.
	 * Default is true.
	 */
	public boolean amongInstancesInThread() default true;
	
	/**
	 * Specify execution order for each event among event listener methods. 
	 * Fore default value for each event, {@link SubscriberMethodUnit#DEFAULT_TURN_VALUE} will be used.  
	 * @return array of what each element corresponds to element of event type array ({@link #value()}) 
	 */
	public int[] orderForEachEvent() default {};
}
